/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file mft_frt.c
 **
 ** A detailed description is available at
 ** @link FrtGroup FRT Module description @endlink
 **
 ** History:
 **   - 2014-01-10  0.1  Edison Zhang  First version.
 **
 *****************************************************************************/


/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/
#include "mft_frt.h"

#if (defined(PDL_PERIPHERAL_MFT_FRT_ACTIVE))
   
/**
 ******************************************************************************
 ** \addtogroup FrtGroup
 ******************************************************************************/
//@{   

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/
/// Look-up table for all enabled FRT of MFT  instances and their internal data
stc_mft_frt_instance_data_t m_astcMftFrtInstanceDataLut[] =
{
#if (PDL_PERIPHERAL_ENABLE_MFT0_FRT == PDL_ON)
    {
        &MFT0_FRT,  // pstcInstance
        {NULL,NULL,NULL,NULL,NULL,NULL}  // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFT1_FRT == PDL_ON)
    {
        &MFT1_FRT,  // pstcInstance
        {NULL,NULL,NULL,NULL,NULL,NULL}  // stcInternData (not initialized yet)
    },
#endif
#if (PDL_PERIPHERAL_ENABLE_MFT2_FRT == PDL_ON)
    {
        &MFT2_FRT,  // pstcInstance
        {NULL,NULL,NULL,NULL,NULL,NULL}  // stcInternData (not initialized yet)
    },
#endif
};

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/
/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/
/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/
/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/
/**
 *****************************************************************************
 ** \brief Return the internal data for a certain FRT instance.
 **
 ** \param pstcFrt Pointer to FRT instance
 **
 ** \return Pointer to internal data or NULL if instance is not enabled
 **         (or not known)
 **
 *****************************************************************************/
static stc_mft_frt_intern_data_t* MftGetInternDataPtr(volatile stc_mftn_frt_t* pstcFrt)
{
    uint8_t u8Instance;

    for (u8Instance = 0; u8Instance < FRT_INSTANCE_COUNT; u8Instance++)
    {
        if (pstcFrt == m_astcMftFrtInstanceDataLut[u8Instance].pstcInstance)
        {
            return &m_astcMftFrtInstanceDataLut[u8Instance].stcInternData;
        }
    }

    return NULL;
}

#if (PDL_INTERRUPT_ENABLE_MFT0_FRT == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_FRT == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_FRT == PDL_ON)
/*!
 ******************************************************************************
 ** \brief  FRT interrupt handler sub function
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  pstcMftFrtInternData  structure of frt callback function
 ** \arg    structure of stc_mft_intern_data
 *****************************************************************************
*/
void Mft_Frt_IrqHandler( volatile stc_mftn_frt_t* pstcFrt,
                         stc_mft_frt_intern_data_t* pstcMftFrtInternData)
{
    static func_ptr_t funCallBack;
    static uint32_t* ptemp;
    uint8_t ch;

    /* Peak match Interrupt */
    for(ch=0;ch<3;ch++)
    {
        if(PdlSet == Mft_Frt_GetIntFlag(pstcFrt,ch,enFrtPeakMatchInt))
        {
            /* Clear Interrupt */
            Mft_Frt_ClrIntFlag(pstcFrt,ch,enFrtPeakMatchInt);

            //get peak match callback address of each channel
            ptemp = (uint32_t*)&(pstcMftFrtInternData->pfnFrt0PeakCallback);
            funCallBack = (func_ptr_t)(*(ptemp + ch));
            
            if(funCallBack != NULL)
            {
               funCallBack();
            }
        }
    }

    /* Zero match Interrupt */
    for(ch=0;ch<3;ch++)
    {
        if(PdlSet == Mft_Frt_GetIntFlag(pstcFrt,ch,enFrtZeroMatchInt))
        {
            /* Clear Interrupt */
            Mft_Frt_ClrIntFlag(pstcFrt,ch,enFrtZeroMatchInt);

            //get Zero match callback address of each channel
            ptemp = (uint32_t*)&(pstcMftFrtInternData->pfnFrt0ZeroCallback);
            funCallBack = (func_ptr_t)(*(ptemp +  ch));
            
            if(funCallBack != NULL)
            {
               funCallBack();
            }
        }
    }
  
}

/**
 ******************************************************************************
 ** \brief Device dependent initialization of interrupts according CMSIS with
 **        level defined in pdl.h
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in] pstcIntSel Pointer to interrupt selection structure
 ******************************************************************************/
static void Mft_Frt_InitIrq( volatile stc_mftn_frt_t* pstcFrt,
                             stc_frt_int_sel_t* pstcIntSel )
{
    if ((pstcFrt == (volatile stc_mftn_frt_t*)(&MFT0_FRT)) || 
        (pstcFrt == (volatile stc_mftn_frt_t*)(&MFT1_FRT)) || 
        (pstcFrt == (volatile stc_mftn_frt_t*)(&MFT2_FRT)))  
    {
        if((pstcIntSel->bFrtZeroMatchInt == 1) || 
           (pstcIntSel->bFrtPeakMatchInt == 1))
        {
            NVIC_ClearPendingIRQ(FRTIM_IRQn);
            NVIC_EnableIRQ(FRTIM_IRQn);
            NVIC_SetPriority(FRTIM_IRQn, PDL_IRQ_LEVEL_MFT_FRT);
        }
    }

    return;
}


/**
 ******************************************************************************
 ** \brief Device dependent initialization of interrupts according CMSIS with
 **        level defined in pdl.h
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in] pstcIntSel Pointer to interrupt selection structure   
 ******************************************************************************/
static void Mft_Frt_DeInitIrq( volatile stc_mftn_frt_t* pstcFrt,
                               stc_frt_int_sel_t* pstcIntSel )
{
    if ((pstcFrt == (volatile stc_mftn_frt_t*)(&MFT0_FRT)) ||
        (pstcFrt == (volatile stc_mftn_frt_t*)(&MFT1_FRT)) || 
        (pstcFrt == (volatile stc_mftn_frt_t*)(&MFT2_FRT)))  
    {
        if((pstcIntSel->bFrtZeroMatchInt == 1) || (pstcIntSel->bFrtPeakMatchInt == 1))
        {
            if(pstcFrt->TCSA0_f.IRQZE == 1)
            {
                return;
            }
            
            if(pstcFrt->TCSA0_f.ICRE == 1)
            {
                return;
            }
            if(pstcFrt->TCSA1_f.IRQZE == 1)
            {
                return;
            }
            
            if(pstcFrt->TCSA1_f.ICRE == 1)
            {
                return;
            }
            if(pstcFrt->TCSA2_f.IRQZE == 1)
            {
                return;
            }
            
            if(pstcFrt->TCSA2_f.ICRE == 1)
            {
                return;
            }
            
            NVIC_ClearPendingIRQ(FRTIM_IRQn);
            NVIC_DisableIRQ(FRTIM_IRQn);
            NVIC_SetPriority(FRTIM_IRQn, PDL_DEFAULT_INTERRUPT_LEVEL);   
                
        }
    }

    return;
}

#endif

/******************************************************************************/
/* Local Functions                                                            */
/******************************************************************************/

/*!
 ******************************************************************************
 ** \brief  Init FRT module
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 ** \param [in] pstcFrtConfig configuration of FRT
 **
 ** \retval Ok                    Internal data has been setup
 ** \retval ErrorInvalidParameter If one of following cases match:
 **                               - pstcMft == NULL
 **                               - pstcFrtConfig == NULL
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_Init(volatile stc_mftn_frt_t* pstcFrt, uint8_t u8Ch,
                         stc_mft_frt_config_t*  pstcFrtConfig)
{
    // Pointer to internal data
    stc_mft_frt_intern_data_t* pstcFrtInternData ;
    volatile stc_mft_frt_tcsa0_field_t* pstcTCSA;

    // Check for NULL pointer and configuration parameter
    if ((pstcFrt == NULL) ||
        (pstcFrtConfig == NULL) ||
        (u8Ch >= MFT_FRT_MAX_CH) ||   
        (pstcFrtConfig->enFrtMode > FrtUpDownCount) ||
        (pstcFrtConfig->enFrtClockDiv > FrtPclkDiv1024) ||
        (pstcFrtConfig->bEnBuffer > 1) ||
        (pstcFrtConfig->bEnExtClock > 1) )
    {
        return ErrorInvalidParameter ;
    }

    // Get pointer to internal data structure ...
    pstcFrtInternData = MftGetInternDataPtr( pstcFrt ) ;
    // ... and check for NULL
    if ( pstcFrtInternData != NULL )
    {
        pstcFrtInternData->pfnFrt0PeakCallback = NULL;
        pstcFrtInternData->pfnFrt0ZeroCallback = NULL;
        pstcFrtInternData->pfnFrt1PeakCallback = NULL;
        pstcFrtInternData->pfnFrt1ZeroCallback = NULL;
        pstcFrtInternData->pfnFrt2PeakCallback = NULL;
        pstcFrtInternData->pfnFrt2ZeroCallback = NULL;
    }

    //Get pointer of current channel frt register address
    pstcTCSA = (volatile stc_mft_frt_tcsa0_field_t*)((volatile uint8_t*)&pstcFrt->TCSA0 + u8Ch*0x0Cu);
    
    //set count clock cycle of FRT counter
    pstcTCSA->CLK = (uint8_t)pstcFrtConfig->enFrtClockDiv;
    
    //set frt mode
    pstcTCSA->MODE = pstcFrtConfig->enFrtMode;

    //set buffer enable bit
    pstcTCSA->BFE = pstcFrtConfig->bEnBuffer;

    //set external clock enable bit
    pstcTCSA->ECKE = pstcFrtConfig->bEnExtClock;

    return Ok;
}
/*!
 ******************************************************************************
 ** \brief  set mask zero times
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer 
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 ** \param [in] u8Times mask times
 ** \arg    value of 0~15
 **
 ** \retval Ok                    Internal data has been setup
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcFrt == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_SetMaskZeroTimes(volatile stc_mftn_frt_t* pstcFrt,
		                             uint8_t u8Ch, uint8_t u8Times)
{
      volatile uint16_t* pu16TCSC;

      // Check for NULL pointer
      if ( (pstcFrt == NULL) || 
           (u8Ch >= MFT_FRT_MAX_CH) || 
           (u8Times > 15) )
      {
          return ErrorInvalidParameter ;
      }

      //Get pointer of current channel frt register address
      pu16TCSC=(volatile uint16_t*)((volatile uint8_t*)pstcFrt+0x4AUL+u8Ch*0x0C);

      *pu16TCSC &= 0xFFF0u;
      *pu16TCSC |= u8Times;
      
      return Ok;
}

/*!
 ******************************************************************************
 ** \brief  get mask zero times
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer 
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \return mask zero times
 **
 *****************************************************************************
*/
uint8_t Mft_Frt_GetMaskZeroTimes(volatile stc_mftn_frt_t* pstcFrt, uint8_t u8Ch)
{
    volatile uint16_t* pu16TCSC;
    uint8_t u8Times;

    // Check for NULL pointer
    if ( (pstcFrt == NULL) 
         || (u8Ch >= MFT_FRT_MAX_CH))
    {
        return 0 ;
    }
		
    //Get pointer of current channel frt register address
    pu16TCSC=(volatile uint16_t*)((volatile uint8_t*)pstcFrt+0x4AUL+u8Ch*0x0C);

    u8Times = (*pu16TCSC & 0x0F00u) >> 8;
    
    return u8Times;
}
/*!
 ******************************************************************************
 ** \brief  set mask peak times
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer 
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 ** \param [in] u8Times mask times
 ** \arg    value of 0~15
 **
 ** \retval Ok                    mask peak times is set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_SetMaskPeakTimes(volatile stc_mftn_frt_t* pstcFrt,
                                     uint8_t u8Ch, uint8_t u8Times)
{
    volatile uint16_t* pu16TCSC;

    // Check for NULL pointer
      if ( (pstcFrt == NULL) || 
           (u8Ch >= MFT_FRT_MAX_CH) || 
           (u8Times > 15) )
      {
          return ErrorInvalidParameter ;
      }
		
    //Get pointer of current channel frt register address
    pu16TCSC=(volatile uint16_t*)((volatile uint8_t*)pstcFrt+0x4AUL+u8Ch*0x0C);
    
    *pu16TCSC &= 0x00F0u;
    *pu16TCSC |= (uint16_t)((uint16_t)u8Times<<4u);
      
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  get mask peak times
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer 
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \return mask peak times
 **
 *****************************************************************************
*/
uint8_t Mft_Frt_GetMaskPeakTimes(volatile stc_mftn_frt_t* pstcFrt, uint8_t u8Ch)
{
    volatile uint16_t* pu16TCSC;
    uint8_t u8Times;

    // Check for NULL pointer
    if ( (pstcFrt == NULL) 
         || (u8Ch >= MFT_FRT_MAX_CH))
    {
        return 0 ;
    }
		
    //Get pointer of current channel frt register address
    pu16TCSC=(volatile uint16_t*)((volatile uint8_t*)pstcFrt+0x4AUL+u8Ch*0x0C);
    
    u8Times = (*pu16TCSC & 0xF000u) >> 12;

    return u8Times;
}

/*!
 ******************************************************************************
 ** \brief  Start FRT
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \retval Ok                    FRT is started
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_Start(volatile stc_mftn_frt_t* pstcFrt, uint8_t u8Ch)
{
    volatile stc_mft_frt_tcsa0_field_t* pstcTCSA;

    // Check for NULL pointer
    if ( (pstcFrt == NULL)
         || (u8Ch >= MFT_FRT_MAX_CH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel frt register address
    pstcTCSA = (volatile stc_mft_frt_tcsa0_field_t*)((volatile uint8_t*)pstcFrt+0x48UL+u8Ch*0x0C);
    pstcTCSA->STOP = 0;
    pstcTCSA->SCLR = 0;
    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  set frt stop
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch   channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \retval Ok                    FRT is stopped
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_Stop(volatile stc_mftn_frt_t* pstcFrt, uint8_t u8Ch)
{
    volatile stc_mft_frt_tcsa0_field_t* pstcTCSA;

    // Check for NULL pointer
    if ( (pstcFrt == NULL)
         || (u8Ch >= MFT_FRT_MAX_CH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel frt register address
    pstcTCSA = (volatile stc_mft_frt_tcsa0_field_t*)((volatile uint8_t*)pstcFrt+0x48UL+u8Ch*0x0C);

    pstcTCSA->STOP = 1;
    pstcTCSA->SCLR = 1;
    return Ok;
}

#if (PDL_INTERRUPT_ENABLE_MFT0_FRT == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT1_FRT == PDL_ON) || \
    (PDL_INTERRUPT_ENABLE_MFT2_FRT == PDL_ON)
/*!
 ******************************************************************************
 ** \brief  enable frt interrupt
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 **
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \param [in]  pstcIntSel  Pointer to interrupt selection structure
 ** \param [in]  pstcFrtIntCallback  Pointer to FRT interrupt callback function
 **
 ** \retval Ok                    FRT interrupt enabled
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_EnableInt(volatile stc_mftn_frt_t* pstcFrt,
                              uint8_t u8Ch,
                              stc_frt_int_sel_t* pstcIntSel,
                              stc_frt_int_cb_t* pstcFrtIntCallback)
{
     volatile stc_mft_frt_tcsa0_field_t* pstcTCSA;
     stc_mft_frt_intern_data_t* pstcFrtInternData ;

    // Check for NULL pointer
    if ( (pstcFrt == NULL)
         || (u8Ch >= MFT_FRT_MAX_CH)
         || (pstcIntSel->bFrtPeakMatchInt > 1)
         || (pstcIntSel->bFrtZeroMatchInt > 1) )
    {
        return ErrorInvalidParameter ;
    }

    // Get pointer to internal data structure ...
    pstcFrtInternData = MftGetInternDataPtr( pstcFrt ) ;

    //Get pointer of current channel frt register address
    pstcTCSA = (volatile stc_mft_frt_tcsa0_field_t*)((volatile uint8_t*)pstcFrt+0x48UL+u8Ch*0x0C);

    if(pstcIntSel->bFrtZeroMatchInt == 1) //zero detect interrupt
    {
        pstcTCSA->IRQZE = 1;
    }
    if(pstcIntSel->bFrtPeakMatchInt == 1) //peak detect interrupt
    {
        pstcTCSA->ICRE = 1;
    }

    //set callback function
    switch(u8Ch)
    {
        case MFT_FRT_CH0: //channel 0
            if(pstcIntSel->bFrtZeroMatchInt == 1) //zero detect interrupt
            {
                pstcFrtInternData->pfnFrt0ZeroCallback = pstcFrtIntCallback->pfnFrtZeroCallback;
            }
            if(pstcIntSel->bFrtPeakMatchInt == 1) //peak detect interrupt
            {
                pstcFrtInternData->pfnFrt0PeakCallback = pstcFrtIntCallback->pfnFrtPeakCallback;
            }
            break;

        case MFT_FRT_CH1: //channel 1
            if(pstcIntSel->bFrtZeroMatchInt == 1) //zero detect interrupt
            {
                pstcFrtInternData->pfnFrt1ZeroCallback = pstcFrtIntCallback->pfnFrtZeroCallback;
            }
            if(pstcIntSel->bFrtPeakMatchInt == 1)  //peak detect interrupt
            {
                pstcFrtInternData->pfnFrt1PeakCallback = pstcFrtIntCallback->pfnFrtPeakCallback;
            }
            break;
        case MFT_FRT_CH2: //channel 2
            if(pstcIntSel->bFrtZeroMatchInt == 1)  //zero detect interrupt
            {
                pstcFrtInternData->pfnFrt2ZeroCallback = pstcFrtIntCallback->pfnFrtZeroCallback;
            }
            if(pstcIntSel->bFrtPeakMatchInt == 1)  //peak detect interrupt
            {
                pstcFrtInternData->pfnFrt2PeakCallback = pstcFrtIntCallback->pfnFrtPeakCallback;
            }
            break;
        default :
            return ErrorInvalidParameter;
    }
    Mft_Frt_InitIrq(pstcFrt,pstcIntSel);

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  disable frt interrupt
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \param [in]  pstcIntSel  Pointer to interrupt selection structure
 **
 ** \retval Ok                    FRT interrupt disabled
 ** \retval ErrorInvalidParameter if one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_DisableInt(volatile stc_mftn_frt_t* pstcFrt, uint8_t u8Ch,
                               stc_frt_int_sel_t* pstcIntSel)
{
    volatile stc_mft_frt_tcsa0_field_t* pstcTCSA;

    // Check for NULL pointer
    if ( (pstcFrt == NULL)
         || (u8Ch >= MFT_FRT_MAX_CH)
         || (pstcIntSel->bFrtPeakMatchInt > 1)
         || (pstcIntSel->bFrtZeroMatchInt > 1))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel frt register address
    pstcTCSA = (volatile stc_mft_frt_tcsa0_field_t*)((volatile uint8_t*)pstcFrt+0x48UL+u8Ch*0x0C);

    if(pstcIntSel->bFrtZeroMatchInt == 1) //zero detect interrupt
    {
        pstcTCSA->IRQZE = 0;
    }
    if(pstcIntSel->bFrtPeakMatchInt == 1) //peak detect interrupt
    {
        pstcTCSA->ICRE = 0;
    }
    Mft_Frt_DeInitIrq(pstcFrt, pstcIntSel);
    return Ok;
}

#endif

/*!
 ******************************************************************************
 ** \brief  get frt interrupt flag
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \param [in]  enIntType  the type of interrupt
 ** \arg     enFrtZeroMatchInt and  enFrtPeakMatchInt
 **
 ** \retval FALSE                 If one of following condition are met:
 **                               - FRT zero match don't occur
 **                               - FRT peak match don't occur
 ** \retval TRUE                  If one of following condition are met:
 **                               - FRT zero match occurs
 **                               - FRT peak match occurs
 **
 *****************************************************************************
*/
en_int_flag_t Mft_Frt_GetIntFlag(volatile stc_mftn_frt_t* pstcFrt,
                                 uint8_t u8Ch,
                                 en_mft_frt_int_t enIntType)
{
    volatile stc_mft_frt_tcsa0_field_t* pstcTCSA;
    en_int_flag_t retval;

    //Get pointer of current channel frt register address
    pstcTCSA = (volatile stc_mft_frt_tcsa0_field_t*)((volatile uint8_t*)pstcFrt+0x48UL+u8Ch*0x0C);

    if(enFrtZeroMatchInt == enIntType) //zero detect interrupt
    {
        (pstcTCSA->IRQZF == 1) ? (retval = PdlSet) : (retval = PdlClr);
    }
    else //peak detect interrupt
    {
        (pstcTCSA->ICLR == 1) ? (retval = PdlSet) : (retval = PdlClr);
    }

    return retval;
}

/*!
 ******************************************************************************
 ** \brief  clear frt interrupt flag
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \param [in]  enIntType  the type of interrupt
 ** \arg     enFrtZeroMatchInt and  enFrtPeakMatchInt
 **
 ** \retval Ok                    frt interrupt flag is clear
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_ClrIntFlag(volatile stc_mftn_frt_t* pstcFrt,
                               uint8_t u8Ch,
                               en_mft_frt_int_t enIntType)
{
    volatile stc_mft_frt_tcsa0_field_t* pstcTCSA;

    // Check for NULL pointer
    if ( (pstcFrt == NULL)
         || (u8Ch >= MFT_FRT_MAX_CH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel frt register address
    pstcTCSA = (volatile stc_mft_frt_tcsa0_field_t*)((volatile uint8_t*)pstcFrt+0x48UL+u8Ch*0x0C);

    if(enFrtZeroMatchInt == enIntType) //zero detect interrupt
    {
        pstcTCSA->IRQZF = 0;
    }
    else //peak detect interrupt
    {
        pstcTCSA->ICLR = 0;
    }

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  set frt cycle value
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \param [in]  u16Cycle  the cycle value
 **
 ** \retval Ok                    FRT cycle value is set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - u8Ch out of range
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_SetCountCycle(volatile stc_mftn_frt_t* pstcFrt,
                                  uint8_t u8Ch, uint16_t u16Cycle)
{
    volatile uint16_t* pu16TCCP;

    // Check for NULL pointer
    if ( (pstcFrt == NULL)
         || (u8Ch >= MFT_FRT_MAX_CH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel frt register address
    pu16TCCP = (volatile uint16_t*)( (volatile uint8_t*)pstcFrt + 0x42UL + u8Ch*0x0C);

    *pu16TCCP = u16Cycle;

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  set frt count value
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \param [in]  u16Count  the count value
 ** \arg     number of 16bit
 **
 ** \retval Ok                    FRT count value is set
 ** \retval ErrorInvalidParameter If one of following conditions are met:
 **                               - pstcMft == NULL
 **                               - Other invalid configuration
 **
 *****************************************************************************
*/
en_result_t Mft_Frt_SetCountVal(volatile stc_mftn_frt_t* pstcFrt,
                                uint8_t u8Ch,uint16_t u16Count)
{
    volatile uint16_t* pu16TCDT;

    // Check for NULL pointer
    if ( (pstcFrt == NULL)
         || (u8Ch >= MFT_FRT_MAX_CH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel frt register address
    pu16TCDT = (volatile uint16_t*)( (volatile uint8_t*)pstcFrt + 0x46UL + u8Ch*0x0C);

    *pu16TCDT = u16Count;

    return Ok;
}

/*!
 ******************************************************************************
 ** \brief  get frt current count
 **
 ** \param [in] pstcFrt Pointer to FRT instance
 ** \param [in]  u8Ch  channel of Free run timer
 ** \arg    MFT_FRT_CH0 ~ MFT_FRT_CH2
 **
 ** \retval current count of frt
 **
 *****************************************************************************
*/
uint16_t Mft_Frt_GetCurCount(volatile stc_mftn_frt_t* pstcFrt, uint8_t u8Ch)
{
    uint16_t retTCDT;
    volatile  uint16_t* pstcTCDT;

    // Check for NULL pointer
    if ( (pstcFrt == NULL)
         || (u8Ch >= MFT_FRT_MAX_CH))
    {
        return ErrorInvalidParameter ;
    }

    //Get pointer of current channel frt register address
    pstcTCDT = (volatile uint16_t*)( (volatile uint8_t*)pstcFrt + 0x46UL + u8Ch*0x0C);

    retTCDT = *pstcTCDT;

    return retTCDT;
}

//@} // FrtGroup

#endif // #if (defined(PDL_PERIPHERAL_$$X_ACTIVE))

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
