/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/*****************************************************************************/
/** \file bootloaderapi.c
 **
 ** API to execute bootloader commands
 **
 ** History:
 **   - 2012-03-16    1.0  MSc  First version
 **   - 2014-07-02    1.1  MSc  Strip down
 *****************************************************************************/


/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/

#include "mcu.h"
#include "bootloaderapi.h"
#include "bootloader.h"


/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

//static void delay(volatile uint32_t u32TimeOut);

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief  Delay loop
 **
 ** \param  u32Timeout Timeout value
 ******************************************************************************/
/*static void delay(volatile uint32_t u32TimeOut)
{
   volatile uint32_t i;
   u32TimeOut = u32TimeOut * 100;
   while(u32TimeOut--)
   {
       for(i=10;i>0;i--)
       {
       }
   }
}*/

/**
 ******************************************************************************
 ** \brief  Jump to user code application
 **
 ** \param  u32Address Address of user code
 ** \return none
 ******************************************************************************/
#ifdef __ICCARM__
  void BootloaderAPI_JumpBoot(uint32_t u32Address)
  {
    #if (__VTOR_PRESENT == 1)
     __asm("LDR SP, [R0]");		//Load new stack pointer address
     __asm("LDR PC, [R0, #4]");	        //Load new program counter address
    #endif
  }
#elif  __CC_ARM
  __asm void BootloaderAPI_JumpBoot(uint32_t u32Address)
  {
		#if (__VTOR_PRESENT == 1)
     LDR SP, [R0]							;Load new stack pointer address
     LDR PC, [R0, #4]	        ;Load new program counter address
		#endif
  }
#else
  #error Please check compiler and linker settings for RAM code
#endif

/**
 ******************************************************************************
 ** \brief  Execute main application
 **
 ** \param  none
 ** \return none
 ******************************************************************************/
void BootloaderAPI_ExecuteUserApplication(void)
{
    #if (__VTOR_PRESENT == 1)
    SCB->VTOR = USER_FLASH_START & 0x1FFFFF80; //Change the Vector Table to the USER_FLASH_START 
    #endif
    BootloaderAPI_JumpBoot(USER_FLASH_START);
}

/**
 ******************************************************************************
 ** \brief  Check user code area is empty
 **
 ** \param  none
 ** \return TRUE, if it is empty
 ******************************************************************************/
boolean_t BootloaderAPI_CheckResetVectorOk(void)
{
    if (((volatile uint32_t*)(USER_FLASH_START + 4))[0] == 0xFFFFFFFF)
    {
        return FALSE;
    }
    return TRUE;
}

/**
 ******************************************************************************
 ** \brief  Execute software reset
 **
 ** \param  none
 ** \return none
 ******************************************************************************/
void BootloaderAPI_ExecuteSoftwareReset(void)
{
    *(volatile unsigned long*)(0xE000ED0C)=0x05FA0004; //SW Reset
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

