/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/*****************************************************************************/
/** \file bootloaderapi.h
 **
 ** API to execute bootloader commands
 **
 ** History:
 **   - 2012-03-16    1.0  MSc  First version
 **   - 2014-07-02    1.1  MSc  Strip down
 *****************************************************************************/

#ifndef __BOOTLOADERAPI_H__
#define __BOOTLOADERAPI_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/

#include "mcu.h"
#include "base_types.h"

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif
  
/**
 ******************************************************************************
 ** \defgroup BootloaderApiGroup Additional bootloader API
 **
 ** Provided functions of Bootloader API module:
 ** - BootloaderAPI_JumpBoot()
 ** - BootloaderAPI_ExecuteUserApplication()
 ** - BootloaderAPI_CheckResetVectorOk()
 ** - BootloaderAPI_ExecuteSoftwareReset()
 **
 ** BootloaderAPI_JumpBoot() jumpes to the user application without chaning the vector table VTOR
 **
 ** BootloaderAPI_ExecuteUserApplication() executes main application with chaning
 ** the vector table VTOR and jumpimg to the user application. User application
 ** start is defined in USER_FLASH_START
 **
 ** BootloaderAPI_CheckResetVectorOk() checks reset vector defined for USER_FLASH_START
 ** is correct
 **
 ** BootloaderAPI_ExecuteSoftwareReset() execute software reset
 **
 ** \note USER_FLASH_START is defined in bootloader.h
 **
 ******************************************************************************/
//@{
  
/******************************************************************************
 * Global definitions
 ******************************************************************************/

/******************************************************************************
 * Global type definitions
 ******************************************************************************/

/******************************************************************************/
/* Global variable declarations ('extern', definition in C source)            */
/******************************************************************************/

/******************************************************************************/
/* Global function prototypes ('extern', definition in C source)              */
/******************************************************************************/  
void BootloaderAPI_JumpBoot(uint32_t u32Address);
void BootloaderAPI_ExecuteUserApplication(void);
boolean_t BootloaderAPI_CheckResetVectorOk(void);
void BootloaderAPI_ExecuteSoftwareReset(void);

//@}
#ifdef __cplusplus
}
#endif

#endif /* __BOOTLOADERAPI_H__ */
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
