/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/*****************************************************************************/
/** \file hexdwl.h
 **
 ** Implements the HEXDWL protocol
 **
 ** History:
 **   - 2014-06-27    1.0  MSc  First version
 *****************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/

#include "hexdwl.h"

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

static hexdwl_sendbuffer_func_ptr_t pfnSendBuffer = NULL;
static hexdwl_sendchar_func_ptr_t pfnSendChar = NULL;
static hexdwl_sendstring_func_ptr_t pfnSendString = NULL;
static hexdwl_erasesector_func_ptr_t pfnEraseSector = NULL;
static hexdwl_writeflash_func_ptr_t pfnWriteFlash = NULL;
static hexdwl_activity_trigger_func_ptr_t pfnTriggerActivity = NULL;
static hexdwl_reboot_func_ptr_t pfnReboot = NULL;
static hexdwl_status_func_ptr_t pfnStartAction = NULL;
static hexdwl_status_func_ptr_t pfnFinished = NULL;
static hexdwl_receive_func_ptr_t pfnReceiveData = NULL;
static uint8_t au8Temp[6];
static uint16_t au16FlashBuffer[256];
static uint8_t u8State = 0;
static volatile boolean_t bNextData = FALSE;
static uint8_t au8RingBufferData[512];
static stc_ringbuffer_t stcRingBuffer = {au8RingBufferData,
                                 sizeof(au8RingBufferData),
                                 0,
                                 0};
/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/


/**
 ******************************************************************************
 ** \brief  Callback for requesting next data for ring buffer
 **
 ** \return TRUE if no more data was received
 ******************************************************************************/
static boolean_t RequestNextData(void)
{
    volatile uint32_t u32Timeout;
    bNextData = FALSE;
    pfnSendChar(0x06);
    u32Timeout = 1000000000;
    while((bNextData == FALSE) && (u32Timeout > 0)) 
    {
        u32Timeout--;
        if (pfnReceiveData != NULL)
        {
            pfnReceiveData();
        }
    }
    if (u32Timeout == 0)
    {
       return TRUE;
    }
    return FALSE;
}

/**
 ******************************************************************************
 ** \brief  Callback for read data from SRecord parser
 **
 ** \param pu8Buffer Buffer
 **
 ** \param u32DataLength Datasize
 **
 ** \param pu32DataRead Pointer of varaible, storing data was read information
 **
 ******************************************************************************/
static void ReadRingBuffer(uint8_t* pu8Buffer, uint32_t u32DataLength,uint32_t* pu32DataRead)
{
    RingBuffer_Read(&stcRingBuffer, pu8Buffer, u32DataLength, pu32DataRead, RequestNextData);
}

/**
 ******************************************************************************
 ** \brief  Init HEXDWL
 **
 ** \param pstcConfig Configuration
 **
 ******************************************************************************/
void HexDwl_Init(stc_hexdwl_config_t* pstcConfig)
{
    SRecordParser_SetReadDataHandle(ReadRingBuffer);
    pfnSendBuffer = pstcConfig->pfnSendBuffer;
    pfnSendChar = pstcConfig->pfnSendChar;
    pfnSendString = pstcConfig->pfnSendString;
    pfnEraseSector = pstcConfig->pfnEraseSector;
    pfnWriteFlash = pstcConfig->pfnWriteFlash;
    pfnTriggerActivity = pstcConfig->pfnTriggerActivity;
    pfnReboot = pstcConfig->pfnReboot;
    pfnStartAction = pstcConfig->pfnStartAction;
    pfnFinished = pstcConfig->pfnFinished;
    pfnReceiveData = pstcConfig->pfnReceiveData;
}

/**
 ******************************************************************************
 ** \brief  Periodic call from main loop
 **
 ******************************************************************************/
void HexDwl_Tick(void)
{
  uint32_t i;
  uint8_t i2;
  uint32_t u32StartAddress;
  uint8_t u8DataLength;
  
  if ((u8State == 2) && (bNextData == FALSE))
  {
    pfnSendChar(0x06);
    u8State = 3;
  }
  if (bNextData)
  {
    u8State++;
    if (u8State == 1)
    {
      //RingBuffer_SetReadToWritePosition(&stcRingBuffer);
    }
    if (u8State > 1)
    {
      u8State = 2;
      u32StartAddress = SRecordParser_ReadLine((uint8_t*)au16FlashBuffer,&u8DataLength);
      if ((u8DataLength == 0) && (SRECORDPARSER_CRCERROR == u32StartAddress))
      {
        bNextData = FALSE;
        pfnSendString(">Error during flash process...\r\n");
      }
      if (u32StartAddress >= 0x4000)
      {
        for(i = 0; i < u8DataLength;i++)
        {
          for(i2 = FlashManagement_GetSectorCount();i2 > 0;i2--)
          {
              if (astcFlashSectors[i2-1].u32Address == (u32StartAddress + i))
              {
                  pfnEraseSector(astcFlashSectors[i2-1].u32Address);
              }
          }
        }
        pfnWriteFlash((uint16_t*)u32StartAddress,(uint16_t*)au16FlashBuffer,u8DataLength / 2); 
        
      }
    }
    bNextData = FALSE;
  }
}

/**
 ******************************************************************************
 ** \brief  Parse received data
 **
 ** \param pu8Buffer Buffer
 **
 ** \param u32DataLength Datasize
 **
 ******************************************************************************/
void HexDwl_ParseReceived(uint8_t* pu8Data, uint32_t u32ReceviedSize)
{
    uint32_t u32Tmp;
    if (u32ReceviedSize == 0) return;
    pfnTriggerActivity(5000);
    if (pu8Data[u32ReceviedSize - 1] == 27)
    {
        pfnTriggerActivity(30000);
        RingBuffer_Init(&stcRingBuffer);
        pfnSendChar('>');
        u8State = 0;
        return;
    }
    if (pu8Data[u32ReceviedSize - 1] == 0x0D)
    {
        bNextData = TRUE;
        u32ReceviedSize -= 1;
    }
    if ((u32ReceviedSize > 0) && (u8State == 0)) pfnSendBuffer(pu8Data,u32ReceviedSize);
    RingBuffer_Add(&stcRingBuffer,pu8Data,u32ReceviedSize,NULL);
    if (bNextData)
    {
        RingBuffer_Add(&stcRingBuffer,(uint8_t*)"\r\n",2,NULL);
        if (u8State == 0)
        {
            pfnSendChar('\r');
            RingBuffer_Read(&stcRingBuffer,&au8Temp[0],6,&u32Tmp,NULL);
            if ((u32Tmp >= 4) && \
                (au8Temp[0] == (uint8_t)'B') && \
                (au8Temp[1] == (uint8_t)'O') && \
                (au8Temp[2] == (uint8_t)'O') && \
                (au8Temp[3] == (uint8_t)'T'))
            {
                pfnSendString("Reboot...\r\n");
                if (pfnFinished)
                {
                    pfnFinished(Ok);
                }
                pfnReboot();
                
            } else if ((au8Temp[0] == (uint8_t)'H') && \
                (au8Temp[1] == (uint8_t)'E') && \
                (au8Temp[2] == (uint8_t)'X') && \
                (au8Temp[3] == (uint8_t)'D') && \
                (au8Temp[4] == (uint8_t)'W') && \
                (au8Temp[5] == (uint8_t)'L') && \
                (u32Tmp >= 6))
            {
                SRecordParser_SetReadDataHandle(ReadRingBuffer);
                if (pfnStartAction)
                {
                    pfnStartAction(Ok);
                }
            } else
            {
                RingBuffer_Init(&stcRingBuffer);
                pfnSendString("Command not recognized\r\n");
                bNextData = FALSE;
            }
        }
        
    }
}
/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
