/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/*****************************************************************************/
/** \file hexdwl.c
 **
 ** Implements the HEXDWL protocol
 **
 ** History:
 **   - 2014-06-27    1.0  MSc  First version
 *****************************************************************************/
#ifndef __HEXDWL_H__
#define __HEXDWL_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/

#include "base_types.h"
#include "bootloader.h"
#include "ringbuffer.h"
#include "srecordparser.h"
#include "flashmanagement.h"

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif
  
/**
 ******************************************************************************
 ** \defgroup HexDwlGroup HexDwl protocol implementation
 **
 ** Provided functions of HexDwl module:
 ** - HexDwl_Init()
 ** - HexDwl_ParseReceived()
 ** - HexDwl_Tick()
 **
 ** HexDwl_Init() is called before executing any other command
 **
 ** HexDwl_ParseReceived() is called from data received routines
 **
 ** HexDwl_Tick() is called periodically from main loop
 **
 ******************************************************************************/
//@{
  
/******************************************************************************
 * Global definitions
 ******************************************************************************/

/******************************************************************************
 * Global type definitions
 ******************************************************************************/
  
typedef uint8_t (*hexdwl_sendbuffer_func_ptr_t)(uint8_t* pu8Buffer,uint32_t u32DataLength);
typedef uint8_t (*hexdwl_sendchar_func_ptr_t)(char_t c);
typedef uint8_t (*hexdwl_sendstring_func_ptr_t)(char_t* pcBuffer);
typedef void (*hexdwl_erasesector_func_ptr_t)(uint32_t u32Address);
typedef void (*hexdwl_writeflash_func_ptr_t)(uint16_t* pu16Address,uint16_t* pu16Data, uint32_t u32Length);
typedef void (*hexdwl_activity_trigger_func_ptr_t)(uint32_t u32NewBootTimeoutMs);
typedef void (*hexdwl_reboot_func_ptr_t)(void);
typedef void (*hexdwl_status_func_ptr_t)(en_result_t enResult);
typedef void (*hexdwl_receive_func_ptr_t)(void);

/******************************************************************************
 ** \brief HexDwl Config
 ******************************************************************************/

typedef struct stc_hexdwl_config
{
    hexdwl_sendbuffer_func_ptr_t pfnSendBuffer;              ///< Send buffer callback
    hexdwl_sendchar_func_ptr_t pfnSendChar;                  ///< Send char callback
    hexdwl_sendstring_func_ptr_t pfnSendString;              ///< Send string callback
    hexdwl_erasesector_func_ptr_t pfnEraseSector;            ///< Erase flash sector callback
    hexdwl_writeflash_func_ptr_t pfnWriteFlash;              ///< Write flash callback
    hexdwl_activity_trigger_func_ptr_t pfnTriggerActivity;   ///< Activity trigger callback
    hexdwl_reboot_func_ptr_t pfnReboot;                      ///< Reboot callback
    hexdwl_status_func_ptr_t pfnStartAction;                 ///< Start action callback
    hexdwl_status_func_ptr_t pfnFinished;                    ///< Finished action callback
    hexdwl_receive_func_ptr_t pfnReceiveData;                ///< Force receive new data
} stc_hexdwl_config_t;

/******************************************************************************/
/* Global variable declarations ('extern', definition in C source)            */
/******************************************************************************/

/******************************************************************************/
/* Global function prototypes ('extern', definition in C source)              */
/******************************************************************************/  

void HexDwl_Init(stc_hexdwl_config_t* pstcConfig);
void HexDwl_ParseReceived(uint8_t* pu8Data, uint32_t u32ReceviedSize);
void HexDwl_Tick(void);

//@}
#ifdef __cplusplus
}
#endif

#endif /* __HEXDWL_H__ */

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
