/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/*****************************************************************************/
/** \file bootloader_main.c
 **
 ** Implements the bootloader
 **
 ** History:
 **   - 2014-06-27    1.0  MSc  First version
 *****************************************************************************/

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/

#include "bootloader.h"

#if (BOOTLOADER_USB_DEVICE_CDC == 1) || (BOOTLOADER_UART == 1) 
    #include "hexdwl.h"
#endif
#if BOOTLOADER_USB_HOST_MASSSTORAGE == 1
    #include "filedwl.h"
#endif    
#if BOOTLOADER_FM4_PDL == 1
    #include "pdl.h"
#endif
#if BOOTLOADER_FM0_PDL == 1
    #include "pdl_header.h"
#endif
#if BOOTLOADER_FM3_L3 == 1
    #include "l3.h"
#endif

/* Includes for USB */
#if (BOOTLOADER_USB_HOST_MASSSTORAGE == 1) || (BOOTLOADER_USB_DEVICE_CDC == 1)
    #include "usb.h"
    #if FM_PERIPHERAL_USB_DEVICE_ENABLED == ON
        #include "usbdevice.h"
    #endif
    #if FM_PERIPHERAL_USB_HOST_ENABLED == ON
        #include "usbhost.h"
        #include "usbhostclasssupervisor.h"
    #endif 
    #if (USBDEVICECDCCOM_ENABLED == ON)
        #include "UsbDeviceCdcCom.h"
    #endif
    #if (USBDEVICEHIDCOM_ENABLED == ON)
        #include "UsbDeviceHidCom.h"
    #endif
    #if (USBHOSTMASSSTORAGE_ENABLED == ON)
        #include "UsbHostMassStorage.h"
    #endif
#endif
/* End of includes for USB */

/******************************************************************************/
/* Local pre-processor symbols/macros ('#define')                             */
/******************************************************************************/

/******************************************************************************/
/* Global variable definitions (declared in header file with 'extern')        */
/******************************************************************************/

/******************************************************************************/
/* Local type definitions ('typedef')                                         */
/******************************************************************************/

/******************************************************************************/
/* Local function prototypes ('static')                                       */
/******************************************************************************/

static void StartAction(en_result_t enResult);

static void FinishAction(en_result_t enResult);

static void ActivityUpdate(uint32_t u32NewBootTimeoutMs);

static void RebootAction(void);

static void FlashEraseSector(uint32_t u32Address);

static void delay(volatile uint32_t u32TimeOut);

static void FlashWriteData(uint16_t* pu16Address,uint16_t* pu16Data, uint32_t u32Length);

#if ((BOOTLOADER_UART == 1) && ((BOOTLOADER_FM4_PDL == 1) || (BOOTLOADER_FM0_PDL == 1)))

#if (BOOTLOADER_FM4_PDL == 1)
static void MfsRxIrqHandler(volatile stc_mfsn_t* pstcMfs, void* pvHandle);
#elif (BOOTLOADER_FM0_PDL == 1)
static void MfsRxIrqHandler(volatile stc_mfsn_uart_t* pstcMfs, void* pvHandle);
#endif

uint8_t MfsUartWriteData(uint8_t* pu8Buffer,uint32_t u32DataLength);
uint8_t MfsUartWriteChar(char_t c);
uint8_t MfsUartWriteString(char_t* pcString);
#endif

void ReceiveDataPolled(void);

/******************************************************************************/
/* Local variable definitions ('static')                                      */
/******************************************************************************/

volatile uint32_t u32BootTimeout;

#if BOOTLOADER_USB_DEVICE_CDC == 1
const stc_hexdwl_config_t stcHexDwlConfig = {UsbDeviceCdcCom_SendBuffer,
                                       UsbDeviceCdcCom_SendByte,
                                       UsbDeviceCdcCom_SendString,
                                       FlashEraseSector,
                                       FlashWriteData,
                                       ActivityUpdate,
                                       RebootAction,
                                       StartAction,
                                       FinishAction,
                                       ReceiveDataPolled
                                      };
#endif

#if ((BOOTLOADER_UART == 1) && ((BOOTLOADER_FM4_PDL == 1) || (BOOTLOADER_FM0_PDL == 1)))
const stc_hexdwl_config_t stcHexDwlConfig = {MfsUartWriteData,
                                       MfsUartWriteChar,
                                       MfsUartWriteString,
                                       FlashEraseSector,
                                       FlashWriteData,
                                       ActivityUpdate,
                                       RebootAction,
                                       StartAction,
                                       FinishAction,
                                       ReceiveDataPolled
                                      };
#endif


#if BOOTLOADER_USB_HOST_MASSSTORAGE == 1
const stc_filedwl_config_t stcFileDwlConfig = {(char_t*)"app.mhx",
                                        FlashEraseSector,
                                        FlashWriteData,
                                        ActivityUpdate,
                                        RebootAction,
                                        StartAction,
                                        FinishAction,
                                      };
#endif

#if ((BOOTLOADER_UART == 1) && ((BOOTLOADER_FM4_PDL == 1) || (BOOTLOADER_FM0_PDL == 1)))
#if (BOOTLOADER_FM4_PDL == 1)
static const stc_mfs_uart_config_t stcMfsUartCfg = {
    115200,         // Baud rate
    MfsUartNormal,  // Normal mode
    MfsParityNone,  // Non parity
    MfsOneStopBit,  // 1 stop bit
    MfsEightBits,   // 8 character bits
    FALSE,          // LSB first
    FALSE,          // NRZ
    FALSE           // Not use Hardware Flow
};
#elif (BOOTLOADER_FM0_PDL == 1)
static const stc_mfs_uart_config_t stcMfsUartCfg = {
    UartNormal,
    115200,         // Baud rate
    UartParityNone,  // Non parity
    UartOneStopBit,  // 1 stop bit
    UartEightBits,   // 8 character bits
    UartDataLsbFirst,          // LSB first
    FALSE,          // NRZ
    FALSE           // Not use Hardware Flow
};
#endif
#endif

/******************************************************************************/
/* Function implementation - global ('extern') and local ('static')           */
/******************************************************************************/

/**
 ******************************************************************************
 ** \brief Simple delay loop implementation
 **
 ** \param u32TimeOut delay value
 **
 ******************************************************************************/
static void delay(volatile uint32_t u32TimeOut)
{
   volatile uint32_t i;
   u32TimeOut = u32TimeOut * 100;
   while(u32TimeOut--)
   {
       for(i=10;i>0;i--)
       {
       }
   }
}

/**
 ******************************************************************************
 ** \brief Callback handle for start action
 **
 ** \param enResult Start condition
 **
 ******************************************************************************/
static void StartAction(en_result_t enResult)
{
}

/**
 ******************************************************************************
 ** \brief Callback handle for finish action
 **
 ** \param enResult Finish condition
 **
 ******************************************************************************/
static void FinishAction(en_result_t enResult)
{
}

/**
 ******************************************************************************
 ** \brief Callback handle for flash erase sector
 **
 ** \param u32Address sector address
 **
 ******************************************************************************/
static void FlashEraseSector(uint32_t u32Address)
{
    //DOGM_SetCursor(1,0);
    //DOGM_Printline(2,"E:");
    //DOGM_Printhex(u32Address,8);
    #if BOOTLOADER_FM4_PDL == 1
        MFlash_SectorErase((uint16_t*)u32Address);
    #elif BOOTLOADER_FM3_L3 == 1
        Flash_Erase(u32Address);
    #elif BOOTLOADER_FM0_PDL == 1
        MFlash_SectorErase((uint16_t*)u32Address);
    #endif    
    delay(1000);
}

/**
 ******************************************************************************
 ** \brief Callback handle for flash write data
 **
 ** \param pu16Address pointer of address 
 **
 ** \param pu16Data pointer to data buffer 
 **
 ** \param u32Length length of data
 **
 ******************************************************************************/
static void FlashWriteData(uint16_t* pu16Address,uint16_t* pu16Data, uint32_t u32Length)
{
    //DOGM_SetCursor(1,0);
    //DOGM_Printline(2,"W:");
    //DOGM_Printhex((uint32_t)pu16Address,8);
    #if BOOTLOADER_FM4_PDL == 1
        MFlash_Write(pu16Address,pu16Data,u32Length,FALSE);
    #elif BOOTLOADER_FM3_L3 == 1
        Flash_WriteBlock16((uint32_t)pu16Address,pu16Data,u32Length);
    #elif BOOTLOADER_FM0_PDL == 1
        MFlash_Write(pu16Address,pu16Data,u32Length);
    #endif    

}

/**
 ******************************************************************************
 ** \brief Callback handle for activity update
 **
 ******************************************************************************/
static void ActivityUpdate(uint32_t u32NewBootTimeoutMs)
{
    if (u32BootTimeout < u32NewBootTimeoutMs * 1500)
    {
        u32BootTimeout = u32NewBootTimeoutMs * 1500;
    }
}

/**
 ******************************************************************************
 ** \brief Callback handle for reboot action
 **
 ******************************************************************************/
static void RebootAction(void)
{
    //DOGM_SetCursor(1,0);
    //DOGM_Printline(16,"Reboot user app ");
    delay(1000);
    ((volatile char_t*)0x200017F0)[0] = 'B';
    ((volatile char_t*)0x200017F0)[1] = 'O';
    ((volatile char_t*)0x200017F0)[2] = 'O';
    ((volatile char_t*)0x200017F0)[3] = 'T';
    #if ((BOOTLOADER_USB_DEVICE_CDC == 1) || (BOOTLOADER_USB_HOST_MASSSTORAGE == 1))
        Usb_SwitchUsb((stc_usbn_t*)&USB0,UsbSwitchAllOff,0);
    #endif
    BootloaderAPI_ExecuteSoftwareReset();
}

#if ((BOOTLOADER_UART == 1) && ((BOOTLOADER_FM4_PDL == 1) || (BOOTLOADER_FM0_PDL == 1)))
/**
 ******************************************************************************
 ** \brief MFS receive interrupt service routine (use Low Level).
 **
 ** \param [in]       pstcMfs       Pointer to MFS instance register area
 ** \param [in,out]   pvHandle      Pointer to internal data
 **                                 This pointer is not used for this sample.
 **
 ******************************************************************************/
#if (BOOTLOADER_FM4_PDL == 1)
static void MfsRxIrqHandler(volatile stc_mfsn_t* pstcMfs, void* pvHandle)
#elif (BOOTLOADER_FM0_PDL == 1)
static void MfsRxIrqHandler(volatile stc_mfsn_uart_t* pstcMfs, void* pvHandle)
#endif
{
    uint16_t         u16Data;
    volatile uint8_t u8Ssr;

    // Check Overrun error
    #if (BOOTLOADER_FM4_PDL == 1)
        u8Ssr = Mfs_GetStatus(pstcMfs, MFS_UART_SSR_RDRF);
    #elif (BOOTLOADER_FM0_PDL == 1)
        u8Ssr = Mfs_Uart_GetStatus(pstcMfs, UartRxFull);
    #endif
    if (u8Ssr == 0)
    {
        return;
    }
    
    u8Ssr = 0;    
    // Check Overrun error
    #if (BOOTLOADER_FM4_PDL == 1)
        u8Ssr = Mfs_GetStatus(pstcMfs, MFS_UART_SSR_ERR);
    #elif (BOOTLOADER_FM0_PDL == 1)
        u8Ssr = Mfs_Uart_GetStatus(pstcMfs, UartFrameError);
        u8Ssr |= Mfs_Uart_GetStatus(pstcMfs, UartOverrunError);
        u8Ssr |= Mfs_Uart_GetStatus(pstcMfs, UartParityError);
    #endif
    
    if (0 != u8Ssr)
    {
        // Clear possible reception errors
        #if (BOOTLOADER_FM4_PDL == 1)
            Mfs_ErrorClear(pstcMfs);
        #elif (BOOTLOADER_FM0_PDL == 1)
            Mfs_Uart_ClrStatus(pstcMfs,UartOverrunError);
        #endif
    }

    // Read data from Read Data Register
    #if (BOOTLOADER_FM4_PDL == 1)
        u16Data = Mfs_ReadData(pstcMfs);
    #elif (BOOTLOADER_FM0_PDL == 1)
        u16Data = Mfs_Uart_ReceiveData(pstcMfs);
    #endif
    
    HexDwl_ParseReceived((uint8_t*)&u16Data,1);
}

/**
 ******************************************************************************
 ** \brief Callback handle for serial write data
 **
 ** \param pu8Buffer Buffer 
 **
 ** \param u32DataLength datalength
 **
 ** \return 1
 **
 ******************************************************************************/
uint8_t MfsUartWriteData(uint8_t* pu8Buffer,uint32_t u32DataLength)
{
    uint32_t i;
    for(i = 0;i < u32DataLength;i++)
    {
      
        #if (BOOTLOADER_FM4_PDL == 1)
            while(Mfs_GetStatus(&MFS0, MFS_UART_SSR_TDRE) == 0);
            Mfs_WriteData(&MFS0, (uint16_t)pu8Buffer[i]);
        #elif (BOOTLOADER_FM0_PDL == 1)
            while(Mfs_Uart_GetStatus(&UART0, UartTxEmpty) == 0);
            Mfs_Uart_SendData(&UART0, (uint16_t)pu8Buffer[i]);
        #endif
    }
    return 1;
}

/**
 ******************************************************************************
 ** \brief Callback handle for serial write char
 **
 ** \param c Char 
 **
 ** \return 1
 **
 ******************************************************************************/
uint8_t MfsUartWriteChar(char_t c)
{
    #if (BOOTLOADER_FM4_PDL == 1)
        while(Mfs_GetStatus(&MFS0, MFS_UART_SSR_TDRE) == 0);
        Mfs_WriteData(&MFS0, c);
    #elif (BOOTLOADER_FM0_PDL == 1)
        while(Mfs_Uart_GetStatus(&UART0, UartTxEmpty) == 0);
        Mfs_Uart_SendData(&UART0, c);
    #endif 
    return 1;
}

/**
 ******************************************************************************
 ** \brief Callback handle for serial write string
 **
 ** \param pcString String 
 **
 ** \return 1
 **
 ******************************************************************************/
uint8_t MfsUartWriteString(char_t* pcString)
{
    while(*pcString != 0)
    {
        #if (BOOTLOADER_FM4_PDL == 1)
            while(Mfs_GetStatus(&MFS0, MFS_UART_SSR_TDRE) == 0);
            Mfs_WriteData(&MFS0, *pcString);
        #elif (BOOTLOADER_FM0_PDL == 1)
            while(Mfs_Uart_GetStatus(&UART0, UartTxEmpty) == 0);
            Mfs_Uart_SendData(&UART0, *pcString);
        #endif 
        pcString++;
    }
    return 1;
}
#endif

void ReceiveDataPolled(void)
{
    #if ((BOOTLOADER_UART == 1) && ((BOOTLOADER_FM4_PDL == 1) || (BOOTLOADER_FM0_PDL == 1)))
        #if (BOOTLOADER_FM4_PDL == 1)
           MfsRxIrqHandler(&MFS0,NULL);
        #elif (BOOTLOADER_FM0_PDL == 1)
           MfsRxIrqHandler(&UART0,NULL);
        #endif
    #endif
}

/**
 ******************************************************************************
 ** \brief Init Bootloader
 **
 ******************************************************************************/
void BootloaderMain_Init(void)
{
    #if BOOTLOADER_USB_DEVICE_CDC == 1
        #if defined(bFM4_GPIO_PDIR6_P60)
        if (bFM4_GPIO_PDIR6_P60 == 0)
        {
            RebootAction();
        }
        #endif
        #if defined(bFM3_GPIO_PDIR6_P0)
        if (bFM3_GPIO_PDIR6_P0 == 0)
        {
            RebootAction();
        }
        #endif
    #endif
        
    #if (BOOTLOADER_USB_DEVICE_CDC == 1) || (BOOTLOADER_USB_HOST_MASSSTORAGE == 1)
        UsbConfig_UsbInit();
    #endif
        
    #if ((BOOTLOADER_UART == 1) && ((BOOTLOADER_FM4_PDL == 1) || (BOOTLOADER_FM0_PDL == 1)))
        #if (BOOTLOADER_FM4_PDL == 1)
            /* SIN Pin */
            FM4_GPIO->ADE &= ~(1 << 17); 
            bFM4_GPIO_PFR2_P21 = 1;
            FM4_GPIO->EPFR07 &= ~(0x03 << 4);
            FM4_GPIO->EPFR07 |= (0x01 << 4);
            
            /* SOT Pin */
            FM4_GPIO->ADE &= ~(1 << 16); 
            bFM4_GPIO_PFR2_P22 = 1;
            FM4_GPIO->EPFR07 &= ~(0x03 << 6);
            FM4_GPIO->EPFR07 |= (0x01 << 6);
            
            (void)Mfs_Uart_DeInit(&MFS0);

            (void)Mfs_Uart_Init(&MFS0, (stc_mfs_uart_config_t *)&stcMfsUartCfg);
            
            //Mfs_SetRxIntCallBack(&MFS0, MfsRxIrqHandler);
            // Clear possible reception errors
            Mfs_ErrorClear(&MFS0);
            // Enable TX function
            Mfs_SetTxEnable(&MFS0, TRUE);
            // Enable RX function
            Mfs_SetRxEnable(&MFS0, TRUE);
            // Enable RX interrupt
            //Mfs_SetRxIntEnable(&MFS0, TRUE);
            // Init reception interrupt
            //Mfs_InitRxIrq(&MFS0);
        #elif (BOOTLOADER_FM0_PDL == 1)
            /* SIN Pin */
            bFM0P_GPIO_PFR2_P21 = 1;
            FM0P_GPIO->EPFR07 &= ~(0x03 << 4);
            FM0P_GPIO->EPFR07 |= (0x01 << 4);
            
            /* SOT Pin */
            FM0P_GPIO->ADE &= ~(1 << 7); 
            bFM0P_GPIO_PFR2_P22 = 1;
            FM0P_GPIO->EPFR07 &= ~(0x03 << 6);
            FM0P_GPIO->EPFR07 |= (0x01 << 6);
            
            (void)Mfs_Uart_DeInit(&UART0);

            (void)Mfs_Uart_Init(&UART0, (stc_mfs_uart_config_t *)&stcMfsUartCfg);
            
            //MfsUartIrqHandlerRx(&UART0, MfsRxIrqHandler);
            // Clear possible reception errors
            Mfs_Uart_ClrStatus(&UART0,UartOverrunError);
            // Enable TX function
            Mfs_Uart_EnableFunc(&UART0,UartTx);
            // Enable RX function
            Mfs_Uart_EnableFunc(&UART0,UartRx);
            
            Mfs_Uart_ClrStatus(&UART0,UartOverrunError);
            // Enable RX interrupt
            //Mfs_SetRxIntEnable(&UART0, TRUE);
            // Init reception interrupt
            //Mfs_InitRxIrq(&UART0);
         #endif
        
        
        HexDwl_Init((stc_hexdwl_config_t*)&stcHexDwlConfig);
    #endif
        
                
    #if BOOTLOADER_USB_DEVICE_CDC == 1
        HexDwl_Init((stc_hexdwl_config_t*)&stcHexDwlConfig);
        UsbDeviceCdcCom_SetReceivedCallback(HexDwl_ParseReceived);
    #endif
        
    #if BOOTLOADER_USB_HOST_MASSSTORAGE == 1
        FileDwl_Init((stc_filedwl_config_t*)&stcFileDwlConfig);
    #endif
    u32BootTimeout = 3000000;
        
    #if (BOOTLOADER_FM0_PDL == 1)
        u32BootTimeout = 1000000;
    #endif
    if (!BootloaderAPI_CheckResetVectorOk())
    {
        u32BootTimeout = 100000000;
    }
}

/**
 ******************************************************************************
 ** \brief Bootloader periodic task
 **
 ******************************************************************************/
void BootloaderMain_Run(void)
{
    for(;;)
    {
        u32BootTimeout--;
        if (u32BootTimeout == 0)
        {
            RebootAction();
        }
        #if (BOOTLOADER_USB_DEVICE_CDC == 1) || (BOOTLOADER_USB_HOST_MASSSTORAGE == 1)
            #if (((USB_USE_PDL == 1) || (USB_USE_L3 == 1) || (USB_USE_EXT_INT == 0)) && (!defined(BOARD_USB) || (BOARD_USB == OFF)))
                UsbConfig_SwitchMode();  // switch USB<n> if required, otherwise, initialize USB host/device mode
            #endif
        #endif
        #if (BOOTLOADER_USB_DEVICE_CDC == 1) || (BOOTLOADER_UART == 1)
            HexDwl_Tick();
        #endif
        #if BOOTLOADER_USB_HOST_MASSSTORAGE == 1
            FileDwl_Tick();
        #endif    
        ReceiveDataPolled();
    }
}

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/



