/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved. 
*
* This software is owned and published by: 
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND 
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion 
* components. This software is licensed by Spansion to be adapted only 
* for use in systems utilizing Spansion components. Spansion shall not be 
* responsible for misuse or illegal use of this software for devices not 
* supported herein.  Spansion is providing this software "AS IS" and will 
* not be responsible for issues arising from incorrect user implementation 
* of the software.  
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS), 
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING, 
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED 
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED 
* WARRANTY OF NONINFRINGEMENT.  
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT, 
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT 
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, 
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR 
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA, 
* SAVINGS OR PROFITS, 
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED 
* FROM, THE SOFTWARE.  
*
* This software may be replicated in part or whole for the licensed use, 
* with the restriction that this Disclaimer and Copyright notice must be 
* included with each copy of this software, whether used in part or whole, 
* at all times.  
*/
/*****************************************************************************/
/** \file bootloader_main.h
 **
 ** Implements the bootloader
 **
 ** History:
 **   - 2014-06-27    1.0  MSc  First version
 *****************************************************************************/
#ifndef __BOOTLOADER_MAIN__
#define __BOOTLOADER_MAIN__

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif
  
/**
 ******************************************************************************
 ** \defgroup BootloaderMainGroup Bootloader Implementation
 **
 ** Provided functions of Bootloader module:
 ** - BootloaderMain_Init()
 ** - BootloaderMain_Run()
 **
 ** BootloaderMain_Init() is called before executing any other command
 **
 ** BootloaderMain_Run() executes the bootloader
 **
 ** \note following data was changed in the startup file for Reset_Handler:
 **   Reset_Handler
 **                 ;0x20047F00 MAGIC WORD ADDRESS IN RAM
 **                 LDR     R0,=0x20047F00
 ** 
 **                 ;IF MAGIC WORD IS BOOT, START DIRECTLY MAIN APPLICATION
 **                 LDRB    r2,[r0,#0x00]
 **                 CMP     R2, #0x42
 **                 BNE.N   Boot_Bootloader
 **                 LDRB    r2,[r0,#0x01]
 **                 CMP     R2, #0x4F
 **                 BNE.N   Boot_Bootloader
 **                 LDRB    r2,[r0,#0x02]
 **                 CMP     R2, #0x4F
 **                 BNE.N   Boot_Bootloader
 **                 LDRB    r2,[r0,#0x03]
 **                 CMP     R2, #0x54
 **                 BNE.N   Boot_Bootloader
 ** 
 **                 ;DELETE MAGIC WORD
 **                 MOVS    r1,#0x00
 **                 STRB    r1,[r0,#0x00] 
 **                 STRB    r1,[r0,#0x01] 
 **                 STRB    r1,[r0,#0x02] 
 **                 STRB    r1,[r0,#0x03] 
 **                 
 **                 ;LOAD START ADDRESS 0x00004000 
 **                 LDR     R0, =0x00004000
 **                 LDR     r2,[r0,#0x04]
 **                 
 **                 ;CHECK IF VALID RESET VECTOR
 **                 CMN.W   r2,#1
 **                 BNE.N   Boot_User_Application
 **                B       Boot_Bootloader
 **                 
 ** Boot_User_Application
 **                 ;RELOCATE VECTOR TABLE 0xE000ED08
 **                 LDR     R1,=0xE000ED08
 **                 STR     r0,[r1,#0x00]
 **                 
 **                 LDR SP, [R0]		//Load new stack pointer address
 **                 LDR PC, [R0, #4]	//Load new program counter address
 **                 
 ** Boot_Bootloader
 **                 LDR     R0, =SystemInit
 **                 BLX     R0
 **                 LDR     R0, =__iar_program_start
 **                 BX      R0
 **
 ******************************************************************************/
//@{
  
/******************************************************************************
 * Global definitions
 ******************************************************************************/

/******************************************************************************
 * Global type definitions
 ******************************************************************************/

/******************************************************************************/
/* Global variable declarations ('extern', definition in C source)            */
/******************************************************************************/

/******************************************************************************/
/* Global function prototypes ('extern', definition in C source)              */
/******************************************************************************/  
  
void BootloaderMain_Init(void);
void BootloaderMain_Run(void);

//@}
#ifdef __cplusplus
}
#endif

#endif /* __BOOTLOADER_MAIN__ */

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/

