/*******************************************************************************
* Copyright (C) 2013 Spansion LLC. All Rights Reserved.
*
* This software is owned and published by:
* Spansion LLC, 915 DeGuigne Dr. Sunnyvale, CA  94088-3453 ("Spansion").
*
* BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND
* BY ALL THE TERMS AND CONDITIONS OF THIS AGREEMENT.
*
* This software contains source code for use with Spansion
* components. This software is licensed by Spansion to be adapted only
* for use in systems utilizing Spansion components. Spansion shall not be
* responsible for misuse or illegal use of this software for devices not
* supported herein.  Spansion is providing this software "AS IS" and will
* not be responsible for issues arising from incorrect user implementation
* of the software.
*
* SPANSION MAKES NO WARRANTY, EXPRESS OR IMPLIED, ARISING BY LAW OR OTHERWISE,
* REGARDING THE SOFTWARE (INCLUDING ANY ACOOMPANYING WRITTEN MATERIALS),
* ITS PERFORMANCE OR SUITABILITY FOR YOUR INTENDED USE, INCLUDING,
* WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY, THE IMPLIED
* WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE OR USE, AND THE IMPLIED
* WARRANTY OF NONINFRINGEMENT.
* SPANSION SHALL HAVE NO LIABILITY (WHETHER IN CONTRACT, WARRANTY, TORT,
* NEGLIGENCE OR OTHERWISE) FOR ANY DAMAGES WHATSOEVER (INCLUDING, WITHOUT
* LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION,
* LOSS OF BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM USE OR
* INABILITY TO USE THE SOFTWARE, INCLUDING, WITHOUT LIMITATION, ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL OR CONSEQUENTIAL DAMAGES OR LOSS OF DATA,
* SAVINGS OR PROFITS,
* EVEN IF SPANSION HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* YOU ASSUME ALL RESPONSIBILITIES FOR SELECTION OF THE SOFTWARE TO ACHIEVE YOUR
* INTENDED RESULTS, AND FOR THE INSTALLATION OF, USE OF, AND RESULTS OBTAINED
* FROM, THE SOFTWARE.
*
* This software may be replicated in part or whole for the licensed use,
* with the restriction that this Disclaimer and Copyright notice must be
* included with each copy of this software, whether used in part or whole,
* at all times.
*/
/******************************************************************************/
/** \file ringbuffer.h
 **
 ** Ring Buffer Implementation
 **
 ** History:
 **   - 2014-06-26  1.0  MSc        First release version.
 **
 ******************************************************************************/

#ifndef __RINGBUFFER_H__
#define __RINGBUFFER_H__

/******************************************************************************/
/* Include files                                                              */
/******************************************************************************/

#include "base_types.h"

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif
  
/**
 ******************************************************************************
 ** \defgroup RingBufferGroup Ring Buffer Implementation
 **
 ** Provided functions of Ring Buffer module:
 ** - RingBuffer_Init()
 ** - RingBuffer_Add()
 ** - RingBuffer_Read()
 ** - RingBuffer_SniffRead()
 ** - RingBuffer_SetReadToWritePosition()
 **
 ** RingBuffer_Init() is called before executing any other command
 **
 ** RingBuffer_Add() is called to add data into the ring buffer
 **
 ** RingBuffer_Read() is called to read data from the ring buffer
 **
 ** RingBuffer_SniffRead() is used read data without touching the ring buffer read pointer
 **
 ** RingBuffer_SetReadToWritePosition() set the read pointer to the current write position
 **
 ******************************************************************************/
//@{
  
/******************************************************************************
 * Global definitions
 ******************************************************************************/

/******************************************************************************
 * Global type definitions
 ******************************************************************************/
  
typedef boolean_t (*ringbuffer_datawait_func_ptr_t)(void);

/******************************************************************************
 ** \brief Ring Buffer Handle
 ******************************************************************************/

typedef struct stc_ringbuffer
{
    uint8_t* pu8Buffer;         ///< Buffer of the ring buffer
    uint32_t u32Size;           ///< Ring Buffer size =sizeof(pu8Buffer)
    uint32_t u32DataWritePos;   ///< Current write position
    uint32_t u32DataReadPos;    ///< Current read position
} stc_ringbuffer_t;

/******************************************************************************/
/* Global variable declarations ('extern', definition in C source)            */
/******************************************************************************/

/******************************************************************************/
/* Global function prototypes ('extern', definition in C source)              */
/******************************************************************************/  

en_result_t RingBuffer_Init(stc_ringbuffer_t* pstcHandle);
en_result_t RingBuffer_Add(stc_ringbuffer_t* pstcHandle, uint8_t* pu8Buffer, uint32_t u32DataLength, ringbuffer_datawait_func_ptr_t pfnOverwriteWaitCallback);
en_result_t RingBuffer_Read(stc_ringbuffer_t* pstcHandle, uint8_t* pu8Buffer, uint32_t u32DataLength,uint32_t* pu32DataRead, ringbuffer_datawait_func_ptr_t pfnDataWaitCallback);
en_result_t RingBuffer_SniffRead(stc_ringbuffer_t* pstcHandle, uint8_t* pu8Buffer, uint32_t u32DataLength,uint32_t* pu32DataRead, ringbuffer_datawait_func_ptr_t pfnDataWaitCallback);
en_result_t RingBuffer_SetReadToWritePosition(stc_ringbuffer_t* pstcHandle);

//@}
#ifdef __cplusplus
}
#endif

#endif /* __RINGBUFFER_H__ */

/******************************************************************************/
/* EOF (not truncated)                                                        */
/******************************************************************************/
